#include <cstdio>
#include <cstring>
#include <iconv.h>
#include <unistd.h>
#include <pqxx/pqxx>
#include <tinyptc.h>
#include "flagtrigger.h"
#include "widestring.h"
#include "fetch_current_tournament.h"
#include "fonts.h"

Tournament active_tournament;
unsigned char framebuf[800 * 600 * 4];

void init(pqxx::connection &conn)
{
	conn.perform(FetchCurrentTournament(&active_tournament));

	if (active_tournament.id == -1) {
		std::fprintf(stderr, "No active tournament\n");
	} else {
		std::fprintf(stderr, "Current tournament is %d (name: '%s')\n",
			active_tournament.id, active_tournament.name.c_str());
	}
}

void main_loop(pqxx::connection &conn)
{
	if (active_tournament.id == -1) {
		// No active tournament, sleep a second or so and exit
		sleep(1);
		return;
	}

	memset(framebuf, 0, 800*600*4);
	
	pqxx::work t(conn, "trx");

	// fetch all songs
	pqxx::result res( t.exec("SELECT * FROM songs WHERE title LIKE 'M%'") );
	unsigned y = 0;
	for (pqxx::result::const_iterator i = res.begin(); i != res.end(); ++i) {
		my_draw_text(i["title"].as(widestring()), framebuf, 0, y, 1, 255, 255, 255);
		y += 20;
//		std::fprintf(stderr, "%s\n", i["title"].c_str());
	}
	t.commit();

	ptc_update(framebuf);
	sleep(1);
}

int main(int argc, char **argv)
{
	ptc_open("CCBS bigscreen", 800, 600);
	
	try {
		init_freetype();
		pqxx::connection conn("dbname=ccbs host=altersex.samfundet.no user=ccbs password=GeT|>>B_");
		FlagTrigger tournament_changed(conn, "active_tournament");
		FlagTrigger rounds_changed(conn, "active_groups");
		
		// when active_tournament or active_rounds is changed, we destroy everything and start from scratch
		// (at least currently)
		for ( ;; ) {
			tournament_changed.reset_flag();
			rounds_changed.reset_flag();
			init(conn);
			do {
				main_loop(conn);
				conn.get_notifs();
			} while (!tournament_changed.get_flag() && !rounds_changed.get_flag());
			std::fprintf(stderr, "active_tournament or active_groups changed, resetting...\n");
		}
	} catch (const std::exception &e) {
		std::fprintf(stderr, "Exception: %s\n", e.what());
		exit(1);
	}
	
	return 0;
}
