#include <vector>
#include <stdexcept>
#include "fonts.h"

std::vector<FT_Face> fonts;

void init_freetype()
{
	FT_Library library;
	FT_Face face;
	if (FT_Init_FreeType(&library))
		throw std::runtime_error("FreeType init failed.");

	// Georgia
	if (FT_New_Face(library, "/usr/share/fonts/truetype/msttcorefonts/Georgia.ttf", 0, &face))
		throw std::runtime_error("Face opening failed.");
	if (FT_Set_Char_Size(face, 0, 12 * 64, 96, 96))
		throw std::runtime_error("Size set failed.");
	fonts.push_back(face);

	// FreeSerif
	if (FT_New_Face(library, "/usr/share/fonts/truetype/freefont/FreeSerif.ttf", 0, &face)) {
		std::fprintf(stderr, "Warning: Couldn't open FreeSerif, some glyphs might not be available\n");
	} else {
		if (FT_Set_Char_Size(face, 0, 12 * 64, 96, 96))
			throw std::runtime_error("Size set failed.");
		fonts.push_back(face);
	}
	
	// Arial Unicode MS
	if (FT_New_Face(library, "arialuni.ttf", 0, &face)) {
		std::fprintf(stderr, "Warning: Couldn't open Arial Unicode MS, some glyphs might not be available\n");
	} else {
		if (FT_Set_Char_Size(face, 0, 12 * 64, 96, 96))
			throw std::runtime_error("Size set failed.");
		fonts.push_back(face);
	}
}

int my_draw_text(const widestring &str, unsigned char *buf, int xpos, int ypos, bool real_render, int r, int g, int b)
{
	FT_GlyphSlot slot;
	int x = 0;

	for (widestring::const_iterator i = str.begin(); i != str.end(); ++i) {
		int glyph_index;
		for (std::vector<FT_Face>::const_iterator j = fonts.begin(); j != fonts.end(); ++j) {
			glyph_index = FT_Get_Char_Index(*j, *i);
			if (glyph_index == 0)
				continue;

			if (FT_Load_Glyph(*j, glyph_index, FT_LOAD_RENDER))
				throw std::runtime_error("Couldn't load glyph");
			slot = (*j)->glyph;
			break;
		}
		if (glyph_index == 0) {
			std::fprintf(stderr, "Warning: Could not find a glyph in any font for U+%x, ignoring\n", *i);
			continue;
		}

		if (real_render) {
			int y;
			FT_Bitmap *bm = &(slot->bitmap);
			for (y = 0; y < bm->rows; y++) {
				int xx;
				int dsty = ypos - slot->bitmap_top + y;
				if (dsty < 0 || dsty > 599) continue;

				unsigned char *dst = buf + dsty * 800*4 + (x + xpos + slot->bitmap_left)*4;
				unsigned char *src = bm->buffer + y * bm->width;
				for (xx = 0; xx < bm->width; xx++) {
					*dst = (*dst * (256-*src) + r * *src) >> 8;
					*dst++;
					*dst = (*dst * (256-*src) + g * *src) >> 8;
					*dst++;
					*dst = (*dst * (256-*src) + b * *src) >> 8;
					*dst++;
					*dst++ = 0;
					src++;
				}
			}
		}

		x += slot->advance.x >> 6;
	}

	return x;
}
